#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import boto3
import json
import time
import traceback

from ecs_crd.defaultJSONEncoder import DefaultJSONEncoder
from ecs_crd.canaryReleaseDeployStep import CanaryReleaseDeployStep
from ecs_crd.destroyGreenStackStep import DestroyGreenStackStep
from ecs_crd.sendNotificationBySnsStep import SendNotificationBySnsStep

class RollbackChangeRoute53WeightsStep(CanaryReleaseDeployStep):

    def __init__(self, infos, logger):
        """initializes a new instance of the class"""
        super().__init__(infos, 'Rollback Route53 Change DNS Weights', logger)

    def _on_execute(self):
        """operation containing the processing performed by this step"""
        try:
            client = boto3.client('route53')
            if self.infos.blue_infos.stack_id !=None:
                self._rollback_weights(client)
            return DestroyGreenStackStep(self.infos, self.logger)
        except Exception as e:
            self.logger.error('RollbackChangeRoute53WeightsStep', exc_info=True)
            self.infos.exit_exception = e
            self.infos.exit_code = 15
            return SendNotificationBySnsStep(self.infos, self.logger)

    def _is_ready_to_rollback_weights(self, fqdn_infos, client):
        response = client.list_resource_record_sets(HostedZoneId=fqdn_infos.hosted_zone_id, StartRecordName=fqdn_infos.name, MaxItems = '2')
        blue = list(filter(lambda x: x['Name'] == f"{fqdn_infos.name}." and x['SetIdentifier'] == self.infos.blue_infos.canary_release, response['ResourceRecordSets']))
        return int(blue[0]['Weight']) < 100

    def _rollback_weights_by_fqdn(self, fqdn_infos, client):
        self._log_information(key='Fqdn', value= fqdn_infos.name)
        response = client.change_resource_record_sets(
            HostedZoneId=self.infos.hosted_zone_id,
            ChangeBatch={
                'Comment': 'Rollback Route53 records sets for canary blue-green deployment',
                'Changes': [
                    {
                        'Action': 'UPSERT',
                        'ResourceRecordSet': {
                            'Name': fqdn_infos.name + '.',
                            'Type': 'CNAME',
                            'SetIdentifier': self.infos.blue_infos.canary_release,
                            'Weight': 100,
                            'TTL': 60,
                            'ResourceRecords': [
                                {
                                    'Value': self.infos.blue_infos.alb_dns
                                },
                            ]
                        }
                    },
                    {
                        'Action': 'UPSERT',
                        'ResourceRecordSet': {
                            'Name': fqdn_infos.name + '.',
                            'Type': 'CNAME',
                            'SetIdentifier': self.infos.green_infos.canary_release,
                            'Weight': 0,
                            'TTL': 60,
                            'ResourceRecords': [
                                {
                                    'Value': self.infos.green_infos.alb_dns
                                },
                            ]
                        }
                    }
                ]
            }
        )
        self._wait(60, 'Change DNS Weights in progress')

    def _rollback_weights(self, client):
        self.logger.info('Blue')
        self.logger.info(f' DNS     :{self.infos.blue_infos.alb_dns}')
        self.logger.info(f' Weight  :100%')
        self.logger.info(f' Release :{self.infos.blue_infos.canary_release}')
        self.logger.info('Green')
        self.logger.info(f' DNS     :{self.infos.green_infos.alb_dns}')
        self.logger.info(f' Weight  :0%')
        self.logger.info(f' Release :{self.infos.green_infos.canary_release}')     
        self.logger.info('')

        for fqdn_infos in self.infos.fqdn:
            self._log_information(key='Fqdn', value=fqdn_infos.name)
            if self._is_ready_to_rollback_weights(fqdn_infos, client):
                self._rollback_weights_by_fqdn(fqdn_infos, client)
            else:
                self.logger.info(f" No change in weight of DNS")

 



#!/usr/bin/env python3
import codecs
import os.path
import re
import sys
from ecs_crd.versionInfos import VersionInfos

from setuptools import setup, find_packages

version_infos = VersionInfos()

setup (
    name = 'ecs-crd-cli',
    author = "Adventiel",
    author_email = "gwendall.garnier.fr@gmail.com",
    include_package_data = True,
    version = version_infos.version,
    description= version_infos.description,
    url='https://github.com/AdventielFr/ecs-crd-cli.git',
    classifiers = [
        "Programming Language :: Python",
        "Development Status :: 1 - Planning",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: French",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.6"
    ],
    install_requires = [
        'bleach==3.1.0',
        'boto3==1.9.220',
        'botocore==1.12.220',
        'certifi==2019.6.16',
        'chardet==3.0.4',
        'Click==7.0',
        'docutils==0.15.2',
        'humanfriendly==4.18',
        'idna==2.8',
        'jmespath==0.9.4',
        'pkginfo==1.5.0.1',
        'Pygments==2.4.2',
        'python-dateutil==2.8.0',
        'PyYAML==5.1.2',
        'readme-renderer==24.0',
        'requests==2.22.0',
        'requests-toolbelt==0.9.1',
        's3transfer==0.2.1',
        'six==1.12.0',
        'termcolor==1.1.0',
        'tqdm==4.35.0',
        'urllib3==1.25.3',
        'webencodings==0.5.1'
    ],
    packages = find_packages(exclude=["tests/*"]),
    package_data = {'': ['ecs_crd/cfn_*_release_deploy.json']},
    scripts = [
        'bin/ecs-crd'
    ]
)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ecs_crd.canaryReleaseDeployStep import CanaryReleaseDeployStep
from ecs_crd.canaryReleaseInfos import ScaleInfos
from ecs_crd.prepareDeploymentContainerDefinitionsStep import PrepareDeploymentContainerDefinitionsStep
from ecs_crd.sendNotificationBySnsStep import SendNotificationBySnsStep

class PrepareDeploymentScaleParametersStep(CanaryReleaseDeployStep):

    def __init__(self, infos, logger):
        """initializes a new instance of the class"""
        super().__init__(infos, f'Prepare {infos.action} ( Scale parameters )', logger)
        self.min_wait = 40

    def _on_execute(self):
        """operation containing the processing performed by this step"""
        try:
            desired = 2
            wait = self.min_wait 
            self.infos.scale_infos = ScaleInfos()
            if 'scale' in self.configuration['canary']:
                scale = self.configuration['canary']['scale']
                if 'desired' in scale:
                    self.infos.scale_infos.desired = int(scale['desired'])
                if 'wait' in scale:
                    self.infos.scale_infos.wait = int(scale['wait'])
            self._log_information(key='Desired  Instances', value=self.infos.scale_infos.desired,indent=1, ljust=18)
            self._log_information(key='Wait', value=f'{self.infos.scale_infos.wait}s', indent=1, ljust=18)
            self.infos.save()
            return PrepareDeploymentContainerDefinitionsStep(self.infos, self.logger)

        except Exception as e:
            self.infos.exit_code = 3
            self.infos.exit_exception = e
            self.logger.error(self.title, exc_info=True)
            return SendNotificationBySnsStep(self.infos, self.logger)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from abc import ABC, abstractmethod
import logging
import yaml
import time
import datetime
import re
import boto3
class CanaryReleaseDeployStep(ABC):

    def __init__(self, infos, title, logger, with_end_log=True, with_start_log=True):
        self.infos = infos
        self.title = title
        self.logger = logger
        self.with_start_log = with_start_log
        self.with_end_log = with_end_log
        self.previous_exit_code = self.infos.exit_code
        self.configuration = self._load_configuration()

    def execute(self):
        if self.with_start_log:
            self._log_start()
        result = self._on_execute()
        if self.with_end_log:
            self._log_end()
        return result

    def _log_start(self):
        self.logger.info(''.ljust(50, '-'))
        self.logger.info(f'Step: {self.title}')
        self.logger.info(''.ljust(50, '-'))

    def _log_sub_title(self, sub_title):
        self.logger.info('')
        self.logger.info(sub_title)
        self.logger.info(''.ljust(50, '-'))

    def _log_information(self, key, value, ljust=None, indent=0):
        if not ljust:
            ljust = len(key)
        data = "{}{}{} {}".format(''.ljust(indent), key.ljust(ljust), ':' if value!=None else '', '' if not value else value)
        self.logger.info(data)

    def _log_end(self):
        self.logger.info('')
        result = 'COMPLETED'
        if self.infos.exit_code != 0 and self.previous_exit_code == 0:
            result = 'FAILED'
        self.logger.info(f'Step Result : {result}')
        self.logger.info('')

    def _to_int(self, val):
        try:
            return int(val)
        except:
            return None

    def _process_property(self, **kwargs):
        origin = None
        source = None
        source_property = None
        target_property = None
        parent_property = None
        type = None
        pattern = None
        required = False
        multi = False
        default = None
        indent = 0
        
        for k,v in kwargs.items():
            if k == 'source':
                source = v
            if k == 'target':
                target = v
            if k == 'source_property':
                source_property = v
            if k == 'target_property':
                target_property = v
            if k == 'type':
                type = v
            if k == 'pattern':
                pattern = v
            if k == 'required':
                required = bool(v)
            if k == 'default':
                default = v
            if k == 'indent':
                indent = int(v)
            if k == 'parent_property':
                parent_property = v
            if k == 'multi':
                multi = bool(v)
        
        if not target_property:
            target_property = self._to_pascal_case(source_property)
        
        if default and source_property not in source:
            source[source_property] = default
        
        suffix_message = '.'
        if parent_property:
            suffix_message = f' for {parent_property}.'

        if source_property in source:
            if type:
                if isinstance(source[source_property], type):
                    target[target_property] = source[source_property]
                else:
                    raise ValueError(f'{target_property}: {source[source_property]} is not valid{suffix_message}')
            else:
                target[target_property] = self._bind_data(source[source_property])
            if pattern:
                val = re.match(pattern, target[target_property])
                if not val:
                    raise ValueError(f'{target_property}: {source[source_property]} is not valid{suffix_message}')
            self._log_information(key = ('- ' if multi else '' )+target_property, value=target[target_property] , indent=indent)
        else:
            if required:
                raise ValueError(f'{target_property} is required{suffix_message}')

    def _bind_data_fqdn(self, source):
        try:
            result = source
            match = re.match('.*({{fqdn(\\[(\\d)\\])*}}).*',source)
            while match != None:
                if match.groups()[2]:
                    index = int(match.groups()[2])
                    pattern = str(match.groups()[0])
                    result = result.replace(pattern, self.infos.fqdn[index].name)
                else:
                    result = result.replace(str(match.groups()[0]),self.infos.fqdn[0].name)
                match = re.match('.*({{fqdn(\\[(\\d)\\])*}}).*',result)
            return result
        except:
            raise ValueError('Invalid Fqdn template :{}'.format(source))

    def _bind_data(self, source):
        if not source:
            return None
        data = source
        if self.infos.account_id:
            data = data.replace('{{account_id}}', self.infos.account_id)
        if self.infos.environment:
            data = data.replace('{{environment}}', self.infos.environment)
        if self.infos.region:
            data = data.replace('{{region}}', self.infos.region)
        if self.infos.project:
            data = data.replace('{{project}}', self.infos.project)
        if self.infos.service_name:
            data = data.replace('{{name}}', self.infos.service_name)
        if self.infos.service_version:
            data = data.replace('{{version}}', self.infos.service_version)
        if self.infos.external_ip:
            data = data.replace('{{external_ip}}', self.infos.external_ip)
        data = self._bind_data_fqdn(data)
        return data

    def _load_configuration(self):
        """load configuration"""
        if self.infos.configuration_file:
            with open(self.infos.configuration_file, 'r') as stream:
                return yaml.safe_load(stream)
        return None

    def _to_snake_case(self, text):
        """convert to snake case"""
        if text:
            str1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', text)
            return re.sub('([a-z0-9])([A-Z])', r'\1_\2', str1).lower()
        return None

    def _to_pascal_case(self,text):
        result = None
        if not text:
            return result
        s = text.split('_')
        return ''.join(map(lambda i: i.capitalize(),s ))

    @abstractmethod
    def _on_execute(self):
        pass

    def _wait(self, wait, label, tick=5):
        t = 0
        while (t < wait):
            time.sleep(tick)
            t += tick
            r = self._second_to_string(t)
            self.logger.info(f'{label} ... [{r} elapsed]')

    def _second_to_string(self, seconds):
        tm = int(seconds)
        day = tm // (24 * 3600)
        tm = tm % (24 * 3600)
        hours = tm // 3600
        tm %= 3600
        minutes = tm // 60
        tm %= 60
        seconds = tm
        result = ''
        if day > 0:
            result += f'{day}d'
        if hours > 0:
            result += f'{day}h'
        if minutes > 0:
            result += f'{minutes}m'
        result += f'{seconds}s'
        return result

    def _generate_name(self, canary_release='', suffix=''):
        env = f'{self.infos.environment}-'
        o = None
        if len(env) > 6:
            o = slice(5)
            env = f'{self.infos.environment[o]}-'
        cr = canary_release
        if len(cr) > 2:
            o = slice(2)
            cr = cr[2:]
        o = slice(64 - (len(env) + len(cr) + len(suffix)+1))
        var = self.infos.service_name[o]
        return f'{env}{var}{suffix}-{cr}'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import boto3
import datetime

from ecs_crd.canaryReleaseDeployStep import CanaryReleaseDeployStep
from ecs_crd.rollbackChangeRoute53WeightsStep import RollbackChangeRoute53WeightsStep
from ecs_crd.destroyBlueStackStep import DestroyBlueStackStep
from ecs_crd.sendNotificationBySnsStep import SendNotificationBySnsStep

class UpdateCanaryReleaseInfoStep(CanaryReleaseDeployStep):

    def __init__(self, infos, logger):
        """initializes a new instance of the class"""
        super().__init__(infos,'Update CanaryRelease Info (dynamo db)', logger)

    def _on_execute(self):
        """operation containing the processing performed by this step"""
        try:
            client = boto3.resource('dynamodb', region_name=self.infos.region)
            table = client.Table('canary_release')
            if self.infos.action == 'deploy':
                if self._exist_item(client, table):
                    self._update_item(table)
                else:
                    self._insert_item(table)
            if self.infos.action == 'undeploy':
                self._delete_item(table)
            return DestroyBlueStackStep(self.infos, self.logger)
        except Exception as e:
            self.logger.error('UpdateCanaryReleaseInfoStep', exc_info=True)
            self.infos.exit_exception = e
            self.infos.exit_code = 4
            return RollbackChangeRoute53WeightsStep(self.infos, self.logger)

    def _exist_item(self, client, table):
        """check if exist item in dynamoDB table"""
        try:
            response = table.get_item(Key={'id': self.infos.get_hash()})
            return True if ('Item' in response and not response['Item']) else False
        except client.exceptions.ResourceNotFoundException:
            return False
        except Exception:
            raise

    def _delete_item(self, table):
        """delete item in dynamo db table """
        table.delete_item(Key={'id': self.infos.get_hash()})

    def _update_item(self, table):
        """update item in dynamo db table """
        table.update_item(
            Key={'id': self.infos.get_hash()},
            UpdateExpression="set service_version=:v, canary_releaset=:c, alb_arn=:a, deploy_at=:d, stack_name=:s",
            ExpressionAttributeValues={
                ':v': self.infos.service_version,
                ':c': self.infos.green_infos.stack['Parameters']['CanaryRelease']['Default'],
                ':a': self.infos.green_infos.stack['Parameters']['LoadBalancer']['Default'],
                ':d': str(datetime.datetime.now().replace(microsecond=0).isoformat()),
                ':s': self.infos.green_infos.stack_name
            },
            ReturnValues="UPDATED_NEW"
        )

    def _insert_item(self, table):
        """insert item in dynamo db table """
        item = {}
        item['id'] = self.infos.get_hash()
        item['canary_group'] = self.infos.canary_group
        item['service_name'] = self.infos.service_name
        item['service_version'] = self.infos.service_version
        item['environment'] = self.infos.environment
        item['project'] = self.infos.project
        item['region'] = self.infos.region
        item['canary_release'] = self.infos.green_infos.stack['Parameters']['CanaryRelease']['Default']
        item['alb_arn'] = self.infos.green_infos.stack['Parameters']['LoadBalancer']['Default']
        item['stack_name'] = self.infos.green_infos.stack_name
        item['deploy_at'] = str(datetime.datetime.now().replace(microsecond=0).isoformat())
        item['ecs_crd_version'] = self.infos.ecs_crd_version
        table.put_item(Item=item)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import boto3

from ecs_crd.canaryReleaseDeployStep import CanaryReleaseDeployStep
from ecs_crd.sendNotificationBySnsStep import SendNotificationBySnsStep
from ecs_crd.destroyStackStep import DestroyStackStep

class DestroyInitStackStep(DestroyStackStep):

    def __init__(self, infos, logger):
        """initializes a new instance of the class"""
        super().__init__(
            infos, 
            'Delete Init Cloudformation Stack', 
            logger,
            infos.init_infos
        )

    def _on_success(self):
        client = boto3.client('route53')
        self._clean_route_53_record_sets(client)
        return SendNotificationBySnsStep(self.infos, self.logger)
        
    def _on_fail(self):
        return SendNotificationBySnsStep(self.infos, self.logger)

    def _find_route_53_record_sets(self, client):
        result = {}
        hosted_zone_ids = list(set(list(map(lambda x: x.hosted_zone_id, self.infos.fqdn))))
        for hosted_zone_id in hosted_zone_ids:
            for k, v in self._find_route_53_record_sets_by_hosted_zone_id(client, hosted_zone_id).items():
                result[k] = v
        return result

    def _find_route_53_record_sets_by_hosted_zone_id(self, client, hosted_zone_id, start_record_name=None, start_record_type=None, start_record_identifier=None):
        result = {}
        client = boto3.client('route53')
        response = None
        if start_record_name:
            if start_record_identifier:
                response = client.list_resource_record_sets( 
                    HostedZoneId = hosted_zone_id, 
                    StartRecordName=start_record_name, 
                    StartRecordType=start_record_type, 
                    StartRecordIdentifier=start_record_identifier)
            else:
                response = client.list_resource_record_sets( 
                    HostedZoneId = hosted_zone_id, 
                    StartRecordName = start_record_name, 
                    StartRecordType = start_record_type)
        else:
            response = client.list_resource_record_sets( 
                HostedZoneId=hosted_zone_id)
        recordsets = response['ResourceRecordSets']
        for item in self.infos.fqdn:
            exist = list(filter(lambda x : x['Name'].strip('.') == item.name,  recordsets))
            if(len(exist)>0):
                result[item.name] = {}
                result[item.name]['ResourceRecordSets'] = exist
                result[item.name]['HostedZoneId'] = item.hosted_zone_id

        if 'IsTruncated' in response and bool(response['IsTruncated']):
            items = self._find_route_53_record_sets(client, hosted_zone_id, response['NextRecordName'], response['NextRecordType'], response['NextRecordIdentifier'] if 'NextRecordIdentifier' in response else None )
            for k, v in items:
                result[k] = v
        return result 

    def _clean_route_53_record_sets(self, client):
        """remove CNAME in route 53"""
        self.logger.info('Deleting Route53 records ... ')
        for k, v in self._find_route_53_record_sets(client).items():
            self.logger.info('')
            self._log_information(key='Fqdn', value=k)
            change_batch = {}
            change_batch['Comment'] = 'Remove Route53 records sets for canary blue-green deployment'
            change_batch['Changes'] = []
            for record in v['ResourceRecordSets']:
                change = {}
                change['Action'] = 'DELETE'
                change['ResourceRecordSet'] = record
                change_batch['Changes'].append(change)
            client.change_resource_record_sets(
                HostedZoneId=v['HostedZoneId'],
                ChangeBatch=change_batch
            )
        self.logger.info('')
        self.logger.info('Route53 records deleted with success.')
     


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import uuid
import datetime
import json
import hashlib

#Attention au try/catch IOError

from ecs_crd.defaultJSONEncoder import DefaultJSONEncoder
from ecs_crd.versionInfos import VersionInfos

class ScaleInfos:
    def __init__(self, **kwargs):
        self.desired = 2
        self.wait = 60
        keys = self.__dict__.keys()
        for k, v in kwargs.items():
            if k in keys:
                self.__dict__[k] = v

class StrategyInfos:
    def __init__(self, **kwargs):
        self.weight = None
        self.wait = None
        keys = self.__dict__.keys()
        for k, v in kwargs.items():
            if k in keys:
                self.__dict__[k] = v

class PolicyInfos:
    def __init__(self, **kwargs):
        self.name = None
        self.effect = None
        self.actions = None
        self.resources = None
        keys = self.__dict__.keys()
        for k, v in kwargs.items():
            if k in keys:
                self.__dict__[k] = v

class StackInfos:
    def __init__(self, **kwargs):
        self.stack_id = None
        self.stack_name = None
        self.stack = None
        keys = self.__dict__.keys()
        for k, v in kwargs.items():
            if k in keys:
                self.__dict__[k] = v

class ReleaseInfos(StackInfos):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.alb_arn = None
        self.alb_dns = None
        self.alb_hosted_zone_id = None
        self.canary_release = None
        keys = self.__dict__.keys()
        for k, v in kwargs.items():
            if k in keys:
                self.__dict__[k] = v

class FqdnInfos(object):
    def __init__(self, **kwargs):
        self.name = None
        self.hosted_zone_name = None
        self.hosted_zone_id = None
        keys = self.__dict__.keys()
        for k, v in kwargs.items():
            if k in keys:
                self.__dict__[k] = v

class LoadBalancerInfos:
    def __init__(self, **kwargs):
        self.arn = None
        self.dns_name = None
        self.canary_release = None
        self.is_elected = False
        self.hosted_zone_id = None
        keys = self.__dict__.keys()
        for k, v in kwargs.items():
            if k in keys:
                self.__dict__[k] = v

class ListenerRuleInfos:
    def __init__(self,  **kwargs):
        self.listener_arn = None
        self.configuration = None
        self.current_priority = 1
        self.listener_arn = None
        keys = self.__dict__.keys()
        for k, v in kwargs.items():
            if k in keys:
                self.__dict__[k] = v

class SecretInfos:
    def __init__(self, **kwargs):
        self.secrets = []
        self.kms_arn = []
        self.secrets_arn = []
        keys = self.__dict__.keys()
        for k, v in kwargs.items():
            if k in keys:
                self.__dict__[k] = v

class CanaryReleaseInfos(object):
    def __init__(self, **kwargs):
        self.id = uuid.uuid4().hex
        self.sns_topic_notification = None
        self.account_id = None
        self.action = None
        self.external_ip = None
        self.exit_code = 0
        self.exit_exception = None
        self.canary_group = None
        self.cluster_name = None
        self.cluster = None
        self.region = None
        self.environment = None
        self.project = None
        self.service_name = None
        self.service_version = None
        self.listener_port = None
        self.fqdn = []
        self.hosted_zone_id = None
        self.vpc_id = None
        self.scale_infos = None
        self.configuration_file = None
        self.strategy_infos = []
        self.init_infos = StackInfos()
        self.init_infos.stack = self._load_init_cloud_formation_template()
        self.green_infos = ReleaseInfos()
        self.green_infos.stack = self._load_green_cloud_formation_template()
        self.blue_infos = None
        self.listener_rules_infos = []
        self.secrets_infos = None
        self.elected_release = None
        self.ecs_crd_version = None

        keys = self.__dict__.keys()
        for k, v in kwargs.items():
            if k in keys:
                self.__dict__[k] = v

    def _load_green_cloud_formation_template(self):
        result = None
        filename = os.path.dirname(os.path.realpath(__file__))+'/cfn_green_release_deploy.json'
        with open(filename, 'r') as file:
            data = file.read()
            result = json.loads(data)
        result['Parameters']['Environment']['Default'] = self.environment
        result['Parameters']['Region']['Default'] = self.region
        return result

    def _load_init_cloud_formation_template(self):
        result = None
        filename = os.path.dirname(os.path.realpath(__file__))+'/cfn_init_release_deploy.json'
        with open(filename, 'r') as file:
            data = file.read()
            result = json.loads(data)
        result['Parameters']['Environment']['Default'] = self.environment
        result['Parameters']['Region']['Default'] = self.region
        return result

    def save(self):
        cache_id = f".deploy-cache/{self.id}"
        if not os.path.exists('.deploy-cache'):
            os.mkdir('.deploy-cache')
        if not os.path.exists(f"{cache_id}"):
            os.mkdir(f".deploy-cache/{self.id}")
        with open(f"{cache_id}/deploy_info.json", 'w') as file:
            file.write(json.dumps(self, cls=DefaultJSONEncoder, indent=4))

    def get_hash(self):
        data = f'{self.canary_group}#{self.service_name}#{self.environment}#{self.region}'
        hash_object = hashlib.md5(data.encode())
        return hash_object.hexdigest()
